@@ -163,9 +163,12 @@
   
 (define (collect-args argd dict used)
   ;;(simple-format #t "collect-args\n")
-  (if (not (eqv? (read-char) #\()) (throw 'parse-error "collect-args"))
+  (if (not (eqv? (skip-ws (read-char)) #\())
+      (let ((fn (or (port-filename (current-input-port)) "(unknown)"))
+	    (ln (1+ (port-line (current-input-port)))))
+	(throw 'parse-error "~A:~A: CPP expecting `('" fn ln)))
   (let iter ((argl (list (scan-cpp-input argd dict used #t))))
-    (simple-format #t "args: ~S\n" argl)
+    ;;(simple-format #t "args: ~S\n" argl)
     (let ((ch (read-char)))
       (if (eqv? ch #\)) (reverse argl)
 	  (iter (cons (scan-cpp-input argd dict used #t) argl))))))
