@@ -1,30 +1,35 @@
 ;;; nyacc/lex.scm
 ;;;
-;;; Copyright (C) 2015 - Matthew R.Wette
+;;; Copyright (C) 2015,2016 - Matthew R.Wette
 ;;; 
-;;; This library is free software; you can redistribute it and/or
-;;; modify it under the terms of the GNU Lesser General Public
-;;; License as published by the Free Software Foundation; either
-;;; version 3 of the License, or (at your option) any later version.
+;;; This library is free software; you can redistribute it and/or modify it
+;;; under the terms of the GNU Lesser General Public License as published by
+;;; the Free Software Foundation; either version 3 of the License, or (at
+;;; your option) any later version.
 ;;;
-;;; This library is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; This library is distributed in the hope that it will be useful, but
+;;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ;;; Lesser General Public License for more details.
 ;;;
-;;; You should have received a copy of the GNU Lesser General Public
-;;; License along with this library; if not, write to the Free Software
-;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>
 
 ;; A module providing procedures for constructing lexical analyzers.
 
 ;; '$fixed '$float '$string '$chlit '$ident
 
 ;; todo: change lexer to return @code{cons-source} instead of @code{cons}
+;; todo: to be fully compliant, C readers need to deal with \ at end of line
 
 ;; todo: figure out what readers return atoms and which pairs
-;; tokens: read-c-ident (methinks)
+;; tokens: read-c-ident 
 ;; pairs: num-reader read-c-num read-c-string
+;; issue: if returning pairs we need this for hashed parsers:
+;;    (define (assc-$ pair) (cons (assq-ref symbols (car pair)) (cdr pair)))
+;; read-comm changed to (read-comm ch bol) where bol is begin-of-line cond
+;; 
+;; read-c-ident 
 
 (define-module (nyacc lex)
   #:export (make-lexer-generator
@@ -44,7 +49,7 @@
 	    like-c-ident?
 	    filter-mt remove-mt map-mt make-ident-like-p 
 	    c:ws c:if c:ir)
-  #:use-module ((srfi srfi-1) #:select (fold remove))
+  #:use-module ((srfi srfi-1) #:select (remove append-reverse))
   #:use-module (ice-9 pretty-print)
   )
 
@@ -76,7 +81,7 @@
 (define c:nx (string->char-set "eEdD"))	; number exponent
 (define c:hx (string->char-set "abcdefABCDEF"))
 
-;; @item eval-reader reader string => result
+;; @deffn eval-reader reader string => result
 ;; For test and debug, this procedure will evaluate a reader on a string.
 ;; A reader is a procedure that accepts a single character argument intended
 ;; to match a specific character sequence.  A reader will read more characters
@@ -88,7 +93,7 @@
   (with-input-from-string string
     (lambda () (reader (read-char)))))
 
-;; @item make-space-skipper chset => proc
+;; @deffn make-space-skipper chset => proc
 ;; This routine will generate a reader to skip whitespace.
 (define (make-space-skipper chset)
   (lambda (ch)
@@ -102,13 +107,13 @@
 	    #t)))
 	#f)))
 	 
-;; @item skip-c-space ch => #f|#t
+;; @deffn skip-c-space ch => #f|#t
 ;; If @code{ch} is space, skip all spaces, then return @code{#t}, else
 ;; return @code{#f}.
 (define skip-c-space (make-space-skipper c:ws))
 
 
-;; @item make-ident-reader cs-first cs-rest => ch -> #f|string
+;; @deffn make-ident-reader cs-first cs-rest => ch -> #f|string
 ;; For identifiers, given the char-set for first character and the char-set
 ;; for following characters, return a return a reader for identifiers.
 ;; The reader takes a character as input and returns @code{#f} or @code{string}.
@@ -126,11 +131,11 @@
 		 (list->string (reverse chl)))))
 	#f)))
 
-;; @item read-c-ident ch => #f|string
+;; @deffn read-c-ident ch => #f|string
 ;; If ident pointer at following char, else (if #f) ch still last-read.
 (define read-c-ident (make-ident-reader c:if c:ir))
 
-;; @item make-ident-like-p ident-reader
+;; @deffn make-ident-like-p ident-reader
 ;; Generate a predicate, from a reader, that determines if a string qualifies
 ;; as an identifier.
 (define (make-like-ident-p reader)
@@ -139,7 +144,7 @@
 (define like-c-ident? (make-like-ident-p read-c-ident))
 
 
-;; @item make-string-reader delim
+;; @deffn make-string-reader delim
 ;; Generate a reader that uses @code{delim} as delimiter for strings.
 ;; TODO: need to handle matlab-type strings.
 ;; TODO: need to handle multiple delim's (like python)
@@ -156,6 +161,8 @@
 		(else (iter (cons ch cl) (read-char)))))
 	#f)))
 
+;; @deffn read-oct ch => "0123"|#f
+;; Read octal number.
 (define read-oct
   (let ((cs:oct (string->char-set "01234567")))
     (lambda (ch)
@@ -169,6 +176,8 @@
 	  (unread-char ch)
 	  cv))))))
 
+;; @deffn read-hex ch => "0x7f"|#f
+;; Read octal number.
 (define read-hex
   (let ((cs:dig (string->char-set "0123456789"))
 	(cs:uhx (string->char-set "ABCDEF"))
@@ -217,12 +226,12 @@
 	      ((eq? ch #\") (cons '$string (list->string (reverse cl))))
 	      (else (iter (cons ch cl) (read-char)))))))
 
-;; @item make-chlit-reader
+;; @deffn make-chlit-reader
 ;; Generate a reader for character literals. NOT DONE.
 ;; For C, this reads @code{'c'} or @code{'\n'}.
 (define (make-chlit-reader . rest) (error "NOT IMPLEMENTED"))
 
-;; @item read-c-chlit ch
+;; @deffn read-c-chlit ch
 ;; @example
 ;; ... 'c' ... => (read-c-chlit #\') => '($ch-lit . #\c)
 ;; @end example
@@ -245,7 +254,7 @@
 		      (else (error "bad escape sequence")))))
 	    (cons '$chlit (string c1))))))
 
-;; @item make-num-reader => (proc ch) => #f|($fixed . "1")|($float . "1.0")
+;; @deffn make-num-reader => (proc ch) => #f|($fixed . "1")|($float . "1.0")
 ;; This routine will clean by adding "0" before or after dot.
 ;; TODO: add arg to specify alternate syntaxes (e.g. "0x123")
 ;; may want to replace "eEdD" w/ "e"
@@ -304,11 +313,11 @@
 	   (unless (eof-object? ch) (unread-char ch))
 	   (cons ty (list->string (reverse chl)))))))))
 
-;; @item read-c-num ch => #f|string
+;; @deffn read-c-num ch => #f|string
 ;; Reader for unsigned numbers as used in C (or close to it).
 (define read-c-num (make-num-reader))
 
-;;.@item si-map string-list ix => a-list
+;;.@deffn si-map string-list ix => a-list
 ;; Convert list of strings to alist of char at ix and strings.
 ;; This is a helper for make-tree.
 (define (si-map string-list ix)
@@ -325,7 +334,7 @@
       (iter (cons (cons (string-ref (car sl) ix) (list (car sl))) sal)
             (cdr sl))))))
 
-;;.@item make-tree strtab -> tree
+;;.@deffn make-tree strtab -> tree
 ;; This routine takes an alist of strings and symbols and makes a tree
 ;; that parses one char at a time and provide @code{'else} entry for
 ;; signaling sequence found.  That is, if @code{("ab" . 1)} is an entry
@@ -340,7 +349,7 @@
 	 (si-map string-list ix)))
   (si-cnvt (map car strtab) 0))
 
-;; @item make-chseq-reader strtab
+;; @deffn make-chseq-reader strtab
 ;; Given alist of pairs (string, token) return a function that eats chars
 ;; until (token . string) is returned or @code{#f} if no match is found.
 (define (make-chseq-reader strtab)
@@ -366,21 +375,34 @@
 	      (pushback (cdr cl))))
 	  #f))))))
 
-;; @item make-comm-reader comm-table [#:eat-newline #t] => lambda
-;; comm-table is list of cons for (start . end) comment
+;; @deffn make-comm-reader comm-table [#:eat-newline #t] => \
+;;   ch bol -> ('$code-comm "..")|('$lone-comm "..")|#f
+;; comm-table is list of cons for (start . end) comment.
 ;; e.g. ("--" "\n") ("/*" "*/")
 ;; test with "/* hello **/"
 ;; If @code{eat-newline} is specified as true then for read comments 
 ;; ending with a newline a newline swallowed with the comment.
-(define* (make-comm-reader comm-table #:key eat-newline)
+;; Note: assumes backslash is never part of the end
+(define* (make-comm-reader comm-table #:key (eat-newline #f))
+
+  (define (mc-read-char)
+    (let ((ch (read-char)))
+      (if (eqv? ch #\\)
+	  (let ((ch (read-char)))
+	    (if (eqv? ch #\newline)
+		(read-char)
+		(begin (unread-char ch) #\\)))
+	  ch)))
+    
   (let ((tree (make-tree comm-table)))
-    (lambda (ch)
+    (lambda (ch bol)
       (letrec
-	  ((match-beg ;; match start of comment, return end-string
+	  ((tval (if bol '$lone-comm '$code-comm))
+	   (match-beg ;; match start of comment, return end-string
 	    (lambda (cl node)
 	      (cond
 	       ((assq-ref node (car cl)) => ;; shift next character
-		(lambda (n) (match-beg (cons (read-char) cl) n)))
+		(lambda (n) (match-beg (cons (mc-read-char) cl) n)))
 	       ((assq-ref node 'else) =>
 		(lambda (res) (unread-char (car cl)) res)) ; yuck?
 	       (else
@@ -397,35 +419,34 @@
 	       ((eq? px (string-length ps))
 		(if (and (not eat-newline) (eq? #\newline (car sl)))
 		    (unread-char #\newline))
-		;;(cons '$comment (list->string (reverse cl))))
-		(if (eqv? (car cl) #\cr) ;; remove trailing \r 
-		    (cons '$comment (list->string (reverse (cdr cl))))
-		    (cons '$comment (list->string (reverse cl)))))
-	       ((null? il) (find-end cl sl (cons (read-char) il) ps px))
+		(if (and (pair? cl) (eqv? (car cl) #\cr)) ;; rem trailing \r 
+		    (cons tval (list->string (reverse (cdr cl))))
+		    (cons tval (list->string (reverse cl)))))
+	       ((null? il) (find-end cl sl (cons (mc-read-char) il) ps px))
 	       ((eof-object? (car il)) (error "open comment"))
-	       ((eq? (car il) (string-ref ps px))
+	       ((eqv? (car il) (string-ref ps px))
 		(find-end cl (cons (car il) sl) (cdr il) ps (1+ px)))
 	       (else
-		(let ((il1 (fold cons il sl)))
+		(let ((il1 (append-reverse sl il)))
 		  (find-end (cons (car il1) cl) '() (cdr il1) ps 0)))))))
 	(let ((ep (match-beg (list ch) tree)))
-	  (if ep (find-end '() '() (list (read-char)) ep 0) #f))))))
+	  (if ep (find-end '() '() (list (mc-read-char)) ep 0) #f))))))
 
 (define read-c-comm (make-comm-reader '(("/*" . "*/") ("//" . "\n"))))
 
-;; @item filter-mt p? al => al
+;; @deffn filter-mt p? al => al
 ;; Filter match-table based on cars of al.
 (define (filter-mt p? al) (filter (lambda (x) (p? (car x))) al))
 
-;; @item remove-mt p? al => al
+;; @deffn remove-mt p? al => al
 ;; Remove match-table based on cars of al.
 (define (remove-mt p? al) (remove (lambda (x) (p? (car x))) al))
 
-;; @item map-mt f al => al
+;; @deffn map-mt f al => al
 ;; Map cars of al.
 (define (map-mt f al) (map (lambda (x) (cons (f (car x)) (cdr x))) al))
 
-;; @item make-lexer-generator match-table => lexer-generator
+;; @deffn make-lexer-generator match-table => lexer-generator
 ;; @example
 ;; (define gen-lexer (make-lexer-generator #:ident-reader my-id-rdr))
 ;; (with-input-from-file "foo" (parse (gen-lexer)))
@@ -454,7 +475,7 @@
 	 (read-num (or num-reader (make-num-reader)))
 	 (read-string (or string-reader (make-string-reader #\")))
 	 (read-chlit (or chlit-reader (lambda (ch) #f)))
-	 (read-comm (or comm-reader (lambda (ch) #f)))
+	 (read-comm (or comm-reader (lambda (ch bol) #f)))
 	 (skip-comm (or comm-skipper (lambda (ch) #f)))
 	 (spaces (or space-chars " \t\r\n"))
 	 (space-cs (cond ((string? spaces) (string->char-set spaces))
@@ -483,11 +504,8 @@
 	     ;;((eq? ch #\newline) (set! bol #t) (iter (read-char)))
 	     ((char-set-contains? space-cs ch) (iter (read-char)))
 	     ((and (eqv? ch #\newline) (set! bol #t) #f))
-	     ((read-comm ch) =>
-	      (lambda (p)
-		(let ((was-bol bol))
-		  (set! bol #f)
-		  (cons (if was-bol '$lone-comm '$code-comm) (cdr p)))))
+	     ((read-comm ch bol) =>
+	      (lambda (p) (set! bol #f) (assc-$ p)))
 	     ((skip-comm ch) (iter (read-char)))
 	     ((read-ident ch) =>
 	      (lambda (s) (or (and=> (assq-ref keytab (string->symbol s))
